CREATE OR REPLACE FUNCTION envers_initial_load()
  RETURNS integer AS
$BODY$
DECLARE
	revnum INTEGER;
	revdate BIGINT;
	row_data_entity_ind tb_entity_independent%ROWTYPE;
	row_data_entity_parent tb_entity_parent%ROWTYPE;
	row_data_entity_child tb_entity_child%ROWTYPE;

BEGIN

	delete from tb_entity_independent_aud;
	delete from tb_entity_parent_aud;
	delete from tb_entity_child_aud;
	delete from revinfo;

	-- 0. Current timestamp to apply to all entities
	revdate := extract('epoch' from CURRENT_TIMESTAMP)::bigint;

	-- 1. independent entity
	FOR row_data_entity_ind IN select * from tb_entity_independent order by id LOOP
		revnum := nextval('hibernate_sequence');
		insert into revinfo values(revnum, revdate);
		insert into tb_entity_independent_aud
			values(row_data_entity_ind.id, revnum, 0, row_data_entity_ind.field1, row_data_entity_ind.field2);
	END LOOP;

	-- 2. parent-child; parent is ManyToOne to the above independent entity
	FOR row_data_entity_parent IN select * from tb_entity_parent order by id LOOP
		revnum := nextval('hibernate_sequence');
		insert into revinfo values(revnum, revdate);
		insert into tb_entity_parent_aud
			values(row_data_entity_parent.id, revnum, 0, row_data_entity_parent.field1, row_data_entity_parent.field2);
		FOR row_data_entity_child IN select * from tb_entity_child where parent_id = row_data_entity_parent.id LOOP
			insert into tb_entity_child_aud
				values(row_data_entity_child.id, revnum, 0, row_data_entity_child.field1, row_data_entity_child.field2);
		END LOOP;
	END LOOP;
	
  return 0;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE;